import { hierarchy } from 'd3';
export const eventSequences = (selection, props) => {
  const {
    data,
    length,
    margin,
    colorScale,
    selectedPath,
    distributions,
    updateSequence,
    selectedSequence,
    selectedSequenceId,
    selectedSequenceIndex,
  } = props;
  
  
  const root = hierarchy(data);
  const descendants = root.descendants();
  const formattedData = data.map(d => hierarchy(d).descendants());
  
  
  // Selection & Enter for Sequences (Rows)
  const sequences = selection
  	.selectAll('.sequences')
  	.data(formattedData, d => d[0].data.data.id);
  const sequencesEnter = sequences
    .enter().append('g')
      .attr('class', 'sequences')
    .merge(sequences)
      .on('mouseover', (d,i) => updateSequence(d,i))
      .attr('transform', (d,i) => 
            `translate(${margin.right}, ${(length + margin.bottom) * i})`);
  sequences.exit().remove();
  
  
  // Selection & Enter for Events (Columns)
  const events = sequencesEnter.selectAll('rect').data((d) => d);
  const eventsEnter = events
    .enter().append('rect')
      .attr('width', length)
      .attr('height', length)
      .attr('stroke', (d) => 
        (d.data.data.CN === undefined)
        ? 'white'
        : colorScale(distributions[d.data.data.CN])
      )
      .attr('stroke-width', '0.5px')
      .attr('x', (d,i) => (margin.right + length) * i)
    .merge(events)
      .attr('fill', (d) => 
        (d.data.data.CN === undefined)
        ? 'white'
        : colorScale(distributions[d.data.data.CN])
      );
}

