import {
  json,
  select,
  timeParse,
  interpolateRdBu,
  scaleSequential,
} from 'd3';
import { eventSequences } from './eventSequences';

const parseDate = timeParse("%Y-%m-%dT%I:%M:%S");
const colorScale = scaleSequential(interpolateRdBu);
const svg = select('#svg').attr('transform', `translate(20, 20)`);
const sequencesG = svg.append('g');

const width = document.body.clientWidth;
const height = document.body.clientHeight;
svg
  .attr('width', width-50)
  .attr('height', height)


// ------ State and Handlers ------ //
let data;
let dataChanged = true;
let endDate;
let startDate;

// filters
let paths;
let selectedPath;
let distributions = {};
let selectedColorValue;
let selectedSequence = [];
let selectedSequenceId = -1;
let selectedSequenceIndex = -1;
const onColorClicked = (value) => { return; };

const updateSequence = (descendants, index) => {
  selectedSequence = Array();
  descendants.forEach((d, i) => selectedSequence.push(d.data.data));
  
  selectedSequenceId = selectedSequence[0].id;
  selectedSequenceIndex = index;
  render();
}

const onPathClicked = (path) => {
  selectedPath = path;
  dataChanged = true;
  render();
}

const applyFilters = () => {
  let filteredData = data;
  if(selectedPath !== 'all') {
    filteredData = filteredData
      .filter(d => d.data.path === selectedPath);
  }
  
  return filteredData;
}

const updateDistributions = (data) => {
  if(dataChanged) {
    distributions = data
      .map(d => d3.hierarchy(d).descendants())
      .reduce((a, currentValue) => {
        currentValue.forEach((d,i) => {
          const val = d.data.data.CN;
          a[val] = (a[val]) ? a[val] + 1 : 1;
        })
        return a;
      }, {});

    delete distributions[undefined];
    dataChanged = false;
  }
}

const render = () => {
  
  let d = [];
  const length = 1;
  const margin = {right: 1, bottom: 0};
  const data = applyFilters();
  updateDistributions(data);
  
  for(let prop in distributions) {
    d.push(distributions[prop]);
  }
  colorScale.domain(d3.extent(d));
  
  sequencesG.attr('transform', 'translate(200, 5)')
  sequencesG.call(eventSequences, {
    data,
    length,
    margin,
    colorScale,
    selectedPath,
    distributions,
    updateSequence,
    selectedSequence,
    selectedSequenceId,
    selectedSequenceIndex,
  });
  
  
  // Reset height of viz div
  const height = svg.node().getBBox().height + 60;
  const content = document.getElementById('viz');
  // content.setAttribute("style", `height:${height};`);
  svg.attr('height', height);
}



json('data.json').then(loadedData => {
  data = loadedData;
  data.forEach(d => d.data.time = parseDate(d.data.time))
  
  // Extract Node CN Distributions
  updateDistributions(data);

  
  // Extract Uniques
  paths = data.map(d => d.data.path).filter((v,i,s) => s.indexOf(v) === i);
  paths.push('all');
  
  // Extract Time Range
  endDate = data[data.length-1].data.time;
  startDate = data[0].data.time;

	// Set Defaults
  selectedPath = 'all';
  
  render();
});

